'use strict';
/**
 * @file
 * @author Andrey Galkin <andrey@futoin.org>
 *
 *
 * Copyright 2014-2017 FutoIn Project (https://futoin.org)
 * Copyright 2014-2017 Andrey Galkin <andrey@futoin.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

var ISync = require('./ISync');

var Mutex = require('./Mutex');

var Throttle = require('./Throttle');
/**
 * Limiter - complex processing limit for AsyncSteps
 */


var Limiter =
/*#__PURE__*/
function (_ISync) {
  _inherits(Limiter, _ISync);

  /**
   * C-tor
   * @param {object} [options={}] - option map
   * @param {integer} [options.concurrent=1]  - maximum concurrent flows
   * @param {integer} [options.max_queue=0] - maximum queued
   * @param {integer} [options.rate=1]  - maximum entries in period
   * @param {integer} [options.period_ms=1000]  - period length
   * @param {integer} [options.burst=0]  - maximum queue for rate limiting
   */
  function Limiter() {
    var _this;

    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Limiter);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(Limiter).call(this));
    _this._mutex = new Mutex(options.concurrent || 1, options.max_queue || 0);
    _this._throttle = new Throttle(options.rate || 1, options.period_ms || 1000, options.burst || 0);
    return _this;
  }

  _createClass(Limiter, [{
    key: "sync",
    value: function sync(as, step, onerror) {
      var _this2 = this;

      as.sync(this._mutex, function (as) {
        as._root._next_args = as._call_args;
        as.sync(_this2._throttle, step, onerror);
      });
    }
  }]);

  return Limiter;
}(ISync);

module.exports = Limiter;
//# sourceMappingURL=Limiter.js.map
