'use strict';
/**
 * @file
 * @author Andrey Galkin <andrey@futoin.org>
 *
 *
 * Copyright 2014-2017 FutoIn Project (https://futoin.org)
 * Copyright 2014-2017 Andrey Galkin <andrey@futoin.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

var ISync = require('./ISync');

var _require = require('./Errors'),
    DefenseRejected = _require.DefenseRejected;

var mtx_sync = function mtx_sync(asp, mtx, step, on_error) {
  var root = asp._root;

  var release_step = function release_step(asi) {
    mtx._release(root);

    root._handle_success(asi._call_args);
  };

  asp._on_cancel = mtx._release_handler;
  asp._queue = [[function (asi) {
    if (mtx._lock(asp, root)) {
      root._handle_success(asp._call_args);
    } else {
      asi.waitExternal();
      asi._call_args = asp._call_args;
    }
  }, undefined], [step, on_error], [release_step, undefined]];
};
/**
 * Mutual exclusion mechanism for AsyncSteps
 */


var Mutex =
/*#__PURE__*/
function (_ISync) {
  _inherits(Mutex, _ISync);

  /**
   * C-tor
   * @param {integer} [max=1] - maximum number of simultaneous critical section entries
   * @param {integer} [max_queue=null] - limit queue length, if set
   */
  function Mutex() {
    var _this;

    var max = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
    var max_queue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

    _classCallCheck(this, Mutex);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(Mutex).call(this));
    _this._max = max;
    _this._locked = 0;
    _this._owners = new WeakMap();
    _this._queue = [];
    _this._max_queue = max_queue;

    _this._release_handler = function (asi) {
      _this._release(asi._root);
    };

    return _this;
  }

  _createClass(Mutex, [{
    key: "_lock",
    value: function _lock(asi, root) {
      var owners = this._owners;
      var owned = owners.get(root);

      if (owned) {
        owners.set(root, owned + 1);
        return true;
      } else if (this._locked >= this._max) {
        var queue = this._queue;
        var max_queue = this._max_queue;

        if (max_queue !== null && queue.length >= max_queue) {
          root.error(DefenseRejected, 'Mutex queue limit');
        }

        queue.push(asi);
        return false;
      } else {
        this._locked += 1;
        owners.set(root, 1);
        return true;
      }
    }
  }, {
    key: "_release",
    value: function _release(root) {
      var owners = this._owners;
      var owned = owners.get(root);

      if (owned) {
        if (owned > 1) {
          owners.set(root, owned - 1);
          return;
        }

        owners["delete"](root);
        this._locked -= 1;
        var queue = this._queue;

        while (queue.length) {
          var other_as = queue.shift();

          if (other_as.state) {
            var other_root = other_as._root;

            this._lock(other_as, other_root);

            other_root._handle_success(other_as._call_args);

            break;
          }
        }
      } else {
        var idx = this._queue.indexOf(root);

        if (idx >= 0) {
          this._queue.splice(idx, 1);
        }
      }
    }
  }, {
    key: "sync",
    value: function sync(as, step, onerror) {
      var _this2 = this;

      as.add(function (as) {
        mtx_sync(as, _this2, step, onerror);
      });
    }
  }]);

  return Mutex;
}(ISync);

module.exports = Mutex;
//# sourceMappingURL=Mutex.js.map
